package com.isu.gaswellwatch.modbus.data.listener;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/23 0:32
 */

import cn.hutool.core.map.MapUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;


@Component
public class DynamicRabbitListener implements ApplicationRunner {

    private static final String GATEWAY_SQL = "SELECT d.id, d.gateway_sn AS identifier FROM device d ORDER BY d.id ";

    private final AmqpAdmin amqpAdmin;
    private final JdbcTemplate jdbcTemplate;
    private final ComposeModbusMessageListener composeListener;
    private final SimpleMessageListenerContainer listenerContainer;

    public DynamicRabbitListener(ConnectionFactory connectionFactory, JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.amqpAdmin = new RabbitAdmin(connectionFactory);
        this.listenerContainer = new SimpleMessageListenerContainer();
        this.listenerContainer.setConnectionFactory(connectionFactory);
        this.listenerContainer.setMessageListener(this.composeListener = new ComposeModbusMessageListener());
        // 启动监听容器
        this.listenerContainer.start();
    }

    @Override
    public void run(ApplicationArguments args) {
        this.composeListener.addBatchMessageListener(new ModbusMessageBackupListener());
        this.composeListener.addBatchMessageListener(new ModbusMessagePersistListener());

        this.jdbcTemplate.queryForList(GATEWAY_SQL).forEach(item -> {
            this.addListenerQueue(String.format(Queues.MODBUS_COLLECT_DATA, MapUtil.getStr(item, "identifier")), null, null);
        });
    }

    public void addListenerQueue(String gatewaySn) {
        this.addListenerQueue(String.format(Queues.MODBUS_COLLECT_DATA, gatewaySn), null, null);
    }

    public void addListenerQueue(String queueName, String exchangeName, String routingKey) {
        Queue queue = QueueBuilder.durable(queueName).build();
        // 声明队列
        this.amqpAdmin.declareQueue(queue);

        if (StringUtils.isNotBlank(exchangeName) && StringUtils.isNotBlank(routingKey)) {
            // 声明直接类型的交换器
            DirectExchange exchange = new DirectExchange(exchangeName);
            this.amqpAdmin.declareExchange(exchange);
            // 绑定队列和交换器
            this.amqpAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(routingKey));
        }

        // 设置监听的队列
        this.listenerContainer.addQueues(queue);
    }

}