package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import com.isu.gaswellwatch.vo.command.knpcv1.Timing;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;

import java.io.Serial;
import java.time.LocalTime;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 20:12
 */
@Getter
@Setter
@SuperBuilder
public class TimerMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 1612870503947873375L;

    /**
     * 定时器1使能开关
     */
    @Builder.Default
    private boolean enable1 = false;
    /**
     * 定时器2使能开关
     */
    @Builder.Default
    private boolean enable2 = false;
    /**
     * 定时器3使能开关
     */
    @Builder.Default
    private boolean enable3 = false;
    /**
     * 定时器4使能开关
     */
    @Builder.Default
    private boolean enable4 = false;

    /**
     * 定时器1开井时间
     */
    @Builder.Default
    private LocalTime openTimer1 = LocalTime.MIN;
    /**
     * 定时器1关井时间
     */
    @Builder.Default
    private LocalTime closeTimer1 = LocalTime.MIN;

    /**
     * 定时器2开井时间
     */
    @Builder.Default
    private LocalTime openTimer2 = LocalTime.MIN;
    /**
     * 定时器2关井时间
     */
    @Builder.Default
    private LocalTime closeTimer2 = LocalTime.MIN;

    /**
     * 定时器3开井时间
     */
    @Builder.Default
    private LocalTime openTimer3 = LocalTime.MIN;
    /**
     * 定时器3关井时间
     */
    @Builder.Default
    private LocalTime closeTimer3 = LocalTime.MIN;

    /**
     * 定时器4开井时间
     */
    @Builder.Default
    private LocalTime openTimer4 = LocalTime.MIN;
    /**
     * 定时器4关井时间
     */
    @Builder.Default
    private LocalTime closeTimer4 = LocalTime.MIN;

    public TimerMode() {
        super(1);
        this.setCode("KNPCV1.RUN.TIMER");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && Objects.nonNull(this.openTimer1) && Objects.nonNull(this.closeTimer1)
                && Objects.nonNull(this.openTimer2) && Objects.nonNull(this.closeTimer2)
                && Objects.nonNull(this.openTimer3) && Objects.nonNull(this.closeTimer3)
                && Objects.nonNull(this.openTimer4) && Objects.nonNull(this.closeTimer4);
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(150);
        command.append("0110006E001C38");
        command.append("000").append(this.enable1 ? "1" : "0");
        command.append("000").append(this.enable2 ? "1" : "0");
        command.append("000").append(this.enable3 ? "1" : "0");
        command.append("000").append(this.enable4 ? "1" : "0");
        command.append(this.toHexString(this.openTimer1)).append(this.toHexString(this.closeTimer1))
                .append(this.toHexString(this.openTimer2)).append(this.toHexString(this.closeTimer2))
                .append(this.toHexString(this.openTimer3)).append(this.toHexString(this.closeTimer3))
                .append(this.toHexString(this.openTimer4)).append(this.toHexString(this.closeTimer4));
        return List.of(ModbusCommandDto.builder()
                .command(command.toString())
                .length(16).build());
    }

}
