package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import com.isu.gaswellwatch.vo.command.knpcv1.Timing;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 20:23
 */
@Getter
@Setter
@SuperBuilder
public class TimingMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 5065742825844725267L;

    /**
     * 计时器开井时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String openTiming = "00:00:00";
    /**
     * 计时器关井时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String closeTiming = "00:00:00";
    /**
     * 计时器延时时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String delayTiming = "00:00:00";

    public TimingMode() {
        super(2);
        this.setCode("KNPCV1.RUN.TIMING");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.openTiming) && StringUtils.isNotBlank(this.closeTiming) && StringUtils.isNotBlank(this.delayTiming);
    }

    public void setOpenTiming(String openTiming) {
        this.openTiming = this.timingValidate(openTiming, "openTiming");
    }

    public void setCloseTiming(String closeTiming) {
        this.closeTiming = this.timingValidate(this.openTiming, "closeTiming");
    }

    public void setDelayTiming(String delayTiming) {
        this.delayTiming = this.timingValidate(delayTiming, "delayTiming");
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(60);
        command.append("01100096000A14")
                .append(this.toHexString(this.openTiming))
                .append(this.toHexString(this.closeTiming))
                .append("0000") // 地址位：0154，值为空
                .append(this.toHexString(this.delayTiming));
        return List.of(ModbusCommandDto.builder()
                .command(command.toString())
                .length(16).build());
    }

}
